package JoelCastellanosAnimations;

import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.PixelWriter;
import javafx.scene.image.WritableImage;
import javafx.scene.paint.Color;

import java.util.Random;

public class Fireworks extends Animation
{
  private Canvas canvas;
  private GraphicsContext gtx;

  private WritableImage backgroundImage;


  private Point point;
  private int delta;
  private int direction;


  private static final int[] dx = {1, 0, -1, 0};
  private static final int[] dy = {0, 1, 0, -1};


  private int step;
  private static final int size = 25;
  private Random random = new Random();
  private int centerX, centerY;
  private int limit = 400;
  private Particle[] firework = new Particle[500];

  private int paletteIdx = 0;


  private static final Color background = Color.rgb(255, 200, 200);
  private PixelWriter pixelWriterForeground;
  private PixelWriter pixelWriterBack;

  public Fireworks(Canvas canvas)
  {
    this.canvas = canvas;
    gtx = canvas.getGraphicsContext2D();

    step = 0;
    delta = 5;
    direction = 0;

    gtx.setLineWidth(3);

    gtx.setFill(background);
    gtx.fillRect(0, 0, DRAW_WIDTH, DRAW_HEIGHT);

    pixelWriterForeground = canvas.getGraphicsContext2D().getPixelWriter();

    backgroundImage = new WritableImage(DRAW_WIDTH, DRAW_HEIGHT);
    pixelWriterBack = backgroundImage.getPixelWriter();


    centerX = DRAW_WIDTH / 2;
    centerY = DRAW_HEIGHT / 2;
    point = new Point(centerX,centerY);


    gtx.setStroke(Color.RED);
    gtx.strokeOval(centerX - limit, centerY - limit, limit * 2, limit * 2);
    gtx.setStroke(Color.BLUE);

    double x = random.nextDouble()*DRAW_WIDTH;
    double y = random.nextDouble()*DRAW_HEIGHT/4;
    for (int i=0; i<firework.length; i++)
    {
      firework[i] = new Particle(x,y);
      double angle = random.nextDouble()*2.0*Math.PI;
      double mag = 1.0/60.0 + random.nextDouble()*(DRAW_WIDTH/2.0)/60.0;
      firework[i].vx = mag*Math.cos(angle);
      firework[i].vy = mag*Math.sin(angle);
    }
  }


  public void handle(long now)
  {

    for (int i = 0; i < 30; i++)
    {
      step++;


      int dx = centerX - point.x;
      int dy = centerY - point.y;

      if (Math.sqrt(dx * dx + dy * dy) >= limit)
      {
        System.out.println(step);
        stop();
        break;
      }
      double r = random.nextDouble();
      if (r < 0.25) { point.y -= 1; }
      else if (r < 0.5) { point.y += 1; }
      else if (r < 0.75) { point.x += 1; }
      else { point.x -= 1;}

      pixelWriterBack.setColor(point.x, point.y,Color.BLUE);
    }

    gtx.drawImage(backgroundImage,0,0);


    gtx.setFill(Color.PALEVIOLETRED);
    for (int i=0; i<firework.length; i++)
    {
      firework[i].vy += Particle.gravity;
      firework[i].x += firework[i].vx;
      firework[i].y += firework[i].vy;
      gtx.fillOval(firework[i].x-5, firework[i].y-5, 10,10);
    }
  }



  //===================================================================================
  //   Point() is a simple data class with public fields and no methods.
  //===================================================================================
  class Point
  {
    public int x, y;

    public Point(int x, int y)
    {
      this.x = x;
      this.y = y;
    }
  }


  //===================================================================================
  //   Particle() is a simple data class with public fields and no methods.
  //===================================================================================
  class Particle
  {
    public double x, y;
    public double vx, vy;
    public Color color;
    public static final double gravity = 0.1;


    public Particle(double x, double y)
    {
      this.x = x;
      this.y = y;
    }
  }

}
